package net.minecraft.client.renderer.texture;

import org.apache.logging.log4j.LogManager;
import com.mojang.realmsclient.RealmsMainScreen;
import java.util.concurrent.CompletionStage;
import java.util.function.Function;
import net.minecraft.client.gui.components.AbstractWidget;
import net.minecraft.client.gui.screens.TitleScreen;
import net.minecraft.util.profiling.ProfilerFiller;
import com.mojang.blaze3d.platform.TextureUtil;
import java.util.Iterator;
import net.minecraft.client.Minecraft;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import javax.annotation.Nullable;
import net.minecraft.CrashReportCategory;
import net.minecraft.ReportedException;
import net.minecraft.CrashReport;
import java.io.IOException;
import com.mojang.blaze3d.systems.RenderSystem;
import com.google.common.collect.Sets;
import com.google.common.collect.Maps;
import net.minecraft.server.packs.resources.ResourceManager;
import java.util.Set;
import java.util.Map;
import net.minecraft.resources.ResourceLocation;
import org.apache.logging.log4j.Logger;
import net.minecraft.server.packs.resources.PreparableReloadListener;

public class TextureManager implements Tickable, PreparableReloadListener {
    private static final Logger LOGGER;
    public static final ResourceLocation INTENTIONAL_MISSING_TEXTURE;
    private final Map<ResourceLocation, AbstractTexture> byPath;
    private final Set<Tickable> tickableTextures;
    private final Map<String, Integer> prefixRegister;
    private final ResourceManager resourceManager;
    
    public TextureManager(final ResourceManager zb) {
        this.byPath = Maps.newHashMap();
        this.tickableTextures = Sets.newHashSet();
        this.prefixRegister = Maps.newHashMap();
        this.resourceManager = zb;
    }
    
    public void bind(final ResourceLocation sm) {
        if (!RenderSystem.isOnRenderThread()) {
            RenderSystem.recordRenderCall(() -> this._bind(sm));
        }
        else {
            this._bind(sm);
        }
    }
    
    private void _bind(final ResourceLocation sm) {
        AbstractTexture dzt3 = this.byPath.get(sm);
        if (dzt3 == null) {
            dzt3 = new SimpleTexture(sm);
            this.register(sm, dzt3);
        }
        dzt3.bind();
    }
    
    public boolean register(final ResourceLocation sm, AbstractTexture dzt) {
        boolean boolean4 = true;
        try {
            dzt.load(this.resourceManager);
        }
        catch (IOException iOException5) {
            if (sm != TextureManager.INTENTIONAL_MISSING_TEXTURE) {
                TextureManager.LOGGER.warn("Failed to load texture: {}", sm, iOException5);
            }
            dzt = MissingTextureAtlasSprite.getTexture();
            this.byPath.put(sm, dzt);
            boolean4 = false;
        }
        catch (Throwable throwable5) {
            final CrashReport h6 = CrashReport.forThrowable(throwable5, "Registering texture");
            final CrashReportCategory i7 = h6.addCategory("Resource location being registered");
            final AbstractTexture dzt2 = dzt;
            i7.setDetail("Resource location", sm);
            i7.setDetail("Texture object class", () -> dzt2.getClass().getName());
            throw new ReportedException(h6);
        }
        this.byPath.put(sm, dzt);
        if (boolean4 && dzt instanceof Tickable) {
            this.tickableTextures.add((Tickable)dzt);
        }
        return boolean4;
    }
    
    @Nullable
    public AbstractTexture getTexture(final ResourceLocation sm) {
        return this.byPath.get(sm);
    }
    
    public ResourceLocation register(final String string, final DynamicTexture dzv) {
        Integer integer4 = this.prefixRegister.get(string);
        if (integer4 == null) {
            integer4 = 1;
        }
        else {
            ++integer4;
        }
        this.prefixRegister.put(string, integer4);
        final ResourceLocation sm5 = new ResourceLocation(String.format("dynamic/%s_%d", string, integer4));
        this.register(sm5, dzv);
        return sm5;
    }
    
    public CompletableFuture<Void> preload(final ResourceLocation sm, final Executor executor) {
        if (!this.byPath.containsKey(sm)) {
            final PreloadedTexture eab4 = new PreloadedTexture(this.resourceManager, sm, executor);
            this.byPath.put(sm, eab4);
            return eab4.getFuture().thenRunAsync(() -> this.register(sm, eab4), TextureManager::execute);
        }
        return CompletableFuture.<Void>completedFuture((Void)null);
    }
    
    private static void execute(final Runnable runnable) {
        Minecraft.getInstance().execute(() -> RenderSystem.recordRenderCall(runnable::run));
    }
    
    @Override
    public void tick() {
        for (final Tickable eai3 : this.tickableTextures) {
            eai3.tick();
        }
    }
    
    public void release(final ResourceLocation sm) {
        final AbstractTexture dzt3 = this.getTexture(sm);
        if (dzt3 != null) {
            TextureUtil.releaseTextureId(dzt3.getId());
        }
    }
    
    @Override
    public CompletableFuture<Void> reload(final PreparationBarrier a, final ResourceManager zb, final ProfilerFiller aim3, final ProfilerFiller aim4, final Executor executor5, final Executor executor6) {
        final Iterator<Map.Entry<ResourceLocation, AbstractTexture>> iterator5;
        Map.Entry<ResourceLocation, AbstractTexture> entry6;
        ResourceLocation sm7;
        AbstractTexture dzt8;
        return CompletableFuture.allOf(TitleScreen.preloadResources(this, executor5), this.preload(AbstractWidget.WIDGETS_LOCATION, executor5)).thenCompose(a::wait).thenAcceptAsync(void3 -> {
            MissingTextureAtlasSprite.getTexture();
            RealmsMainScreen.updateTeaserImages(this.resourceManager);
            iterator5 = this.byPath.entrySet().iterator();
            while (iterator5.hasNext()) {
                entry6 = iterator5.next();
                sm7 = entry6.getKey();
                dzt8 = entry6.getValue();
                if (dzt8 == MissingTextureAtlasSprite.getTexture() && !sm7.equals(MissingTextureAtlasSprite.getLocation())) {
                    iterator5.remove();
                }
                else {
                    dzt8.reset(this, zb, sm7, executor6);
                }
            }
        }, runnable -> RenderSystem.recordRenderCall(runnable::run));
    }
    
    static {
        LOGGER = LogManager.getLogger();
        INTENTIONAL_MISSING_TEXTURE = new ResourceLocation("");
    }
}
