package net.minecraft.client.sounds;

import com.google.common.collect.Sets;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.MarkerManager;
import com.mojang.blaze3d.audio.SoundBuffer;
import java.util.stream.Stream;
import javax.annotation.Nullable;
import com.mojang.math.Vector3f;
import net.minecraft.client.Camera;
import net.minecraft.util.Mth;
import net.minecraft.world.phys.Vec3;
import java.util.function.Consumer;
import com.mojang.blaze3d.audio.Channel;
import java.util.Collection;
import java.util.Iterator;
import net.minecraft.sounds.SoundEvent;
import net.minecraft.core.Registry;
import com.google.common.collect.Lists;
import com.google.common.collect.HashMultimap;
import com.google.common.collect.Maps;
import java.util.concurrent.Executor;
import net.minecraft.server.packs.resources.ResourceManager;
import net.minecraft.client.resources.sounds.Sound;
import net.minecraft.client.resources.sounds.TickableSoundInstance;
import java.util.List;
import net.minecraft.sounds.SoundSource;
import com.google.common.collect.Multimap;
import net.minecraft.client.resources.sounds.SoundInstance;
import java.util.Map;
import com.mojang.blaze3d.audio.Listener;
import com.mojang.blaze3d.audio.Library;
import net.minecraft.client.Options;
import net.minecraft.resources.ResourceLocation;
import java.util.Set;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.Marker;

public class SoundEngine {
    private static final Marker MARKER;
    private static final Logger LOGGER;
    private static final Set<ResourceLocation> ONLY_WARN_ONCE;
    private final SoundManager soundManager;
    private final Options options;
    private boolean loaded;
    private final Library library;
    private final Listener listener;
    private final SoundBufferLibrary soundBuffers;
    private final SoundEngineExecutor executor;
    private final ChannelAccess channelAccess;
    private int tickCount;
    private final Map<SoundInstance, ChannelAccess.ChannelHandle> instanceToChannel;
    private final Multimap<SoundSource, SoundInstance> instanceBySource;
    private final List<TickableSoundInstance> tickingSounds;
    private final Map<SoundInstance, Integer> queuedSounds;
    private final Map<SoundInstance, Integer> soundDeleteTime;
    private final List<SoundEventListener> listeners;
    private final List<TickableSoundInstance> queuedTickableSounds;
    private final List<Sound> preloadQueue;
    
    public SoundEngine(final SoundManager edw, final Options dbp, final ResourceManager zb) {
        this.library = new Library();
        this.listener = this.library.getListener();
        this.executor = new SoundEngineExecutor();
        this.channelAccess = new ChannelAccess(this.library, this.executor);
        this.instanceToChannel = Maps.newHashMap();
        this.instanceBySource = HashMultimap.create();
        this.tickingSounds = Lists.newArrayList();
        this.queuedSounds = Maps.newHashMap();
        this.soundDeleteTime = Maps.newHashMap();
        this.listeners = Lists.newArrayList();
        this.queuedTickableSounds = Lists.newArrayList();
        this.preloadQueue = Lists.newArrayList();
        this.soundManager = edw;
        this.options = dbp;
        this.soundBuffers = new SoundBufferLibrary(zb);
    }
    
    public void reload() {
        SoundEngine.ONLY_WARN_ONCE.clear();
        for (final SoundEvent aah3 : Registry.SOUND_EVENT) {
            final ResourceLocation sm4 = aah3.getLocation();
            if (this.soundManager.getSoundEvent(sm4) == null) {
                SoundEngine.LOGGER.warn("Missing sound for event: {}", Registry.SOUND_EVENT.getKey(aah3));
                SoundEngine.ONLY_WARN_ONCE.add(sm4);
            }
        }
        this.destroy();
        this.loadLibrary();
    }
    
    private synchronized void loadLibrary() {
        if (this.loaded) {
            return;
        }
        try {
            this.library.init();
            this.listener.reset();
            this.listener.setGain(this.options.getSoundSourceVolume(SoundSource.MASTER));
            this.soundBuffers.preload(this.preloadQueue).thenRun(this.preloadQueue::clear);
            this.loaded = true;
            SoundEngine.LOGGER.info(SoundEngine.MARKER, "Sound engine started");
        }
        catch (RuntimeException runtimeException2) {
            SoundEngine.LOGGER.error(SoundEngine.MARKER, "Error starting SoundSystem. Turning off sounds & music", (Throwable)runtimeException2);
        }
    }
    
    private float getVolume(final SoundSource aaj) {
        if (aaj == null || aaj == SoundSource.MASTER) {
            return 1.0f;
        }
        return this.options.getSoundSourceVolume(aaj);
    }
    
    public void updateCategoryVolume(final SoundSource aaj, final float float2) {
        if (!this.loaded) {
            return;
        }
        if (aaj == SoundSource.MASTER) {
            this.listener.setGain(float2);
            return;
        }
        final float float3;
        final float volume;
        this.instanceToChannel.forEach((ecw, a) -> {
            float3 = this.calculateVolume(ecw);
            a.execute(cwp -> {
                if (volume <= 0.0f) {
                    cwp.stop();
                }
                else {
                    cwp.setVolume(volume);
                }
            });
        });
    }
    
    public void destroy() {
        if (this.loaded) {
            this.stopAll();
            this.soundBuffers.clear();
            this.library.cleanup();
            this.loaded = false;
        }
    }
    
    public void stop(final SoundInstance ecw) {
        if (this.loaded) {
            final ChannelAccess.ChannelHandle a3 = this.instanceToChannel.get(ecw);
            if (a3 != null) {
                a3.execute(Channel::stop);
            }
        }
    }
    
    public void stopAll() {
        if (this.loaded) {
            this.executor.flush();
            this.instanceToChannel.values().forEach(a -> a.execute(Channel::stop));
            this.instanceToChannel.clear();
            this.channelAccess.clear();
            this.queuedSounds.clear();
            this.tickingSounds.clear();
            this.instanceBySource.clear();
            this.soundDeleteTime.clear();
            this.queuedTickableSounds.clear();
        }
    }
    
    public void addEventListener(final SoundEventListener edv) {
        this.listeners.add(edv);
    }
    
    public void removeEventListener(final SoundEventListener edv) {
        this.listeners.remove(edv);
    }
    
    public void tick(final boolean boolean1) {
        if (!boolean1) {
            this.tickNonPaused();
        }
        this.channelAccess.scheduleTick();
    }
    
    private void tickNonPaused() {
        ++this.tickCount;
        this.queuedTickableSounds.forEach(this::play);
        this.queuedTickableSounds.clear();
        for (final TickableSoundInstance ecx3 : this.tickingSounds) {
            ecx3.tick();
            if (ecx3.isStopped()) {
                this.stop(ecx3);
            }
            else {
                final float float4 = this.calculateVolume(ecx3);
                final float float5 = this.calculatePitch(ecx3);
                final Vec3 cvi6 = new Vec3(ecx3.getX(), ecx3.getY(), ecx3.getZ());
                final ChannelAccess.ChannelHandle a7 = this.instanceToChannel.get(ecx3);
                if (a7 == null) {
                    continue;
                }
                final float volume;
                final float pitch;
                final Vec3 selfPosition;
                a7.execute(cwp -> {
                    cwp.setVolume(volume);
                    cwp.setPitch(pitch);
                    cwp.setSelfPosition(selfPosition);
                    return;
                });
            }
        }
        final Iterator<Map.Entry<SoundInstance, ChannelAccess.ChannelHandle>> iterator2 = this.instanceToChannel.entrySet().iterator();
        while (iterator2.hasNext()) {
            final Map.Entry<SoundInstance, ChannelAccess.ChannelHandle> entry3 = iterator2.next();
            final ChannelAccess.ChannelHandle a8 = entry3.getValue();
            final SoundInstance ecw5 = entry3.getKey();
            final float float6 = this.options.getSoundSourceVolume(ecw5.getSource());
            if (float6 <= 0.0f) {
                a8.execute(Channel::stop);
                iterator2.remove();
            }
            else {
                if (!a8.isStopped()) {
                    continue;
                }
                final int integer7 = this.soundDeleteTime.get(ecw5);
                if (integer7 > this.tickCount) {
                    continue;
                }
                final int integer8 = ecw5.getDelay();
                if (ecw5.isLooping() && integer8 > 0) {
                    this.queuedSounds.put(ecw5, this.tickCount + integer8);
                }
                iterator2.remove();
                SoundEngine.LOGGER.debug(SoundEngine.MARKER, "Removed channel {} because it's not playing anymore", a8);
                this.soundDeleteTime.remove(ecw5);
                try {
                    this.instanceBySource.remove(ecw5.getSource(), ecw5);
                }
                catch (RuntimeException ex) {}
                if (!(ecw5 instanceof TickableSoundInstance)) {
                    continue;
                }
                this.tickingSounds.remove(ecw5);
            }
        }
        final Iterator<Map.Entry<SoundInstance, Integer>> iterator3 = this.queuedSounds.entrySet().iterator();
        while (iterator3.hasNext()) {
            final Map.Entry<SoundInstance, Integer> entry4 = iterator3.next();
            if (this.tickCount >= entry4.getValue()) {
                final SoundInstance ecw5 = entry4.getKey();
                if (ecw5 instanceof TickableSoundInstance) {
                    ((TickableSoundInstance)ecw5).tick();
                }
                this.play(ecw5);
                iterator3.remove();
            }
        }
    }
    
    public boolean isActive(final SoundInstance ecw) {
        return this.loaded && ((this.soundDeleteTime.containsKey(ecw) && this.soundDeleteTime.get(ecw) <= this.tickCount) || this.instanceToChannel.containsKey(ecw));
    }
    
    public void play(final SoundInstance ecw) {
        if (!this.loaded) {
            return;
        }
        final WeighedSoundEvents edx3 = ecw.resolve(this.soundManager);
        final ResourceLocation sm4 = ecw.getLocation();
        if (edx3 == null) {
            if (SoundEngine.ONLY_WARN_ONCE.add(sm4)) {
                SoundEngine.LOGGER.warn(SoundEngine.MARKER, "Unable to play unknown soundEvent: {}", sm4);
            }
            return;
        }
        if (!this.listeners.isEmpty()) {
            for (final SoundEventListener edv6 : this.listeners) {
                edv6.onPlaySound(ecw, edx3);
            }
        }
        if (this.listener.getGain() <= 0.0f) {
            SoundEngine.LOGGER.debug(SoundEngine.MARKER, "Skipped playing soundEvent: {}, master volume was zero", sm4);
            return;
        }
        final Sound ect5 = ecw.getSound();
        if (ect5 == SoundManager.EMPTY_SOUND) {
            if (SoundEngine.ONLY_WARN_ONCE.add(sm4)) {
                SoundEngine.LOGGER.warn(SoundEngine.MARKER, "Unable to play empty soundEvent: {}", sm4);
            }
            return;
        }
        final float float6 = ecw.getVolume();
        final float float7 = Math.max(float6, 1.0f) * ect5.getAttenuationDistance();
        final SoundSource aaj8 = ecw.getSource();
        final float float8 = this.calculateVolume(ecw);
        final float float9 = this.calculatePitch(ecw);
        final SoundInstance.Attenuation a11 = ecw.getAttenuation();
        final boolean boolean12 = ecw.isRelative();
        if (float8 == 0.0f && !ecw.canStartSilent()) {
            SoundEngine.LOGGER.debug(SoundEngine.MARKER, "Skipped playing sound {}, volume was zero.", ect5.getLocation());
            return;
        }
        final boolean boolean13 = ecw.isLooping() && ecw.getDelay() == 0;
        final Vec3 cvi14 = new Vec3(ecw.getX(), ecw.getY(), ecw.getZ());
        final ChannelAccess.ChannelHandle a12 = this.channelAccess.createHandle(ect5.shouldStream() ? Library.Pool.STREAMING : Library.Pool.STATIC);
        SoundEngine.LOGGER.debug(SoundEngine.MARKER, "Playing sound {} for event {}", ect5.getLocation(), sm4);
        this.soundDeleteTime.put(ecw, this.tickCount + 20);
        this.instanceToChannel.put(ecw, a12);
        this.instanceBySource.put(aaj8, ecw);
        final float pitch;
        final float volume;
        final SoundInstance.Attenuation attenuation;
        final float float10;
        final boolean looping;
        final Vec3 selfPosition;
        final boolean relative;
        a12.execute(cwp -> {
            cwp.setPitch(pitch);
            cwp.setVolume(volume);
            if (attenuation == SoundInstance.Attenuation.LINEAR) {
                cwp.linearAttenuation(float10);
            }
            else {
                cwp.disableAttenuation();
            }
            cwp.setLooping(looping);
            cwp.setSelfPosition(selfPosition);
            cwp.setRelative(relative);
            return;
        });
        if (!ect5.shouldStream()) {
            this.soundBuffers.getCompleteBuffer(ect5.getPath()).thenAccept(cwu -> a12.execute(cwp -> {
                cwp.attachStaticBuffer(cwu);
                cwp.play();
            }));
        }
        else {
            this.soundBuffers.getStream(ect5.getPath()).thenAccept(edp -> a12.execute(cwp -> {
                cwp.attachBufferStream(edp);
                cwp.play();
            }));
        }
        if (ecw instanceof TickableSoundInstance) {
            this.tickingSounds.add((TickableSoundInstance)ecw);
        }
    }
    
    public void queueTickingSound(final TickableSoundInstance ecx) {
        this.queuedTickableSounds.add(ecx);
    }
    
    public void requestPreload(final Sound ect) {
        this.preloadQueue.add(ect);
    }
    
    private float calculatePitch(final SoundInstance ecw) {
        return Mth.clamp(ecw.getPitch(), 0.5f, 2.0f);
    }
    
    private float calculateVolume(final SoundInstance ecw) {
        return Mth.clamp(ecw.getVolume() * this.getVolume(ecw.getSource()), 0.0f, 1.0f);
    }
    
    public void pause() {
        if (this.loaded) {
            this.channelAccess.executeOnChannels(stream -> stream.forEach(Channel::pause));
        }
    }
    
    public void resume() {
        if (this.loaded) {
            this.channelAccess.executeOnChannels(stream -> stream.forEach(Channel::unpause));
        }
    }
    
    public void playDelayed(final SoundInstance ecw, final int integer) {
        this.queuedSounds.put(ecw, this.tickCount + integer);
    }
    
    public void updateSource(final Camera daz) {
        if (!this.loaded || !daz.isInitialized()) {
            return;
        }
        final Vec3 cvi3 = daz.getPosition();
        final Vector3f e4 = daz.getLookVector();
        final Vector3f e5 = daz.getUpVector();
        final Vec3 listenerPosition;
        final Vector3f e6;
        final Vector3f e7;
        this.executor.execute(() -> {
            this.listener.setListenerPosition(listenerPosition);
            this.listener.setListenerOrientation(e6, e7);
        });
    }
    
    public void stop(@Nullable final ResourceLocation sm, @Nullable final SoundSource aaj) {
        if (aaj != null) {
            for (final SoundInstance ecw5 : this.instanceBySource.get(aaj)) {
                if (sm == null || ecw5.getLocation().equals(sm)) {
                    this.stop(ecw5);
                }
            }
        }
        else if (sm == null) {
            this.stopAll();
        }
        else {
            for (final SoundInstance ecw5 : this.instanceToChannel.keySet()) {
                if (ecw5.getLocation().equals(sm)) {
                    this.stop(ecw5);
                }
            }
        }
    }
    
    public String getDebugString() {
        return this.library.getDebugString();
    }
    
    static {
        MARKER = MarkerManager.getMarker("SOUNDS");
        LOGGER = LogManager.getLogger();
        ONLY_WARN_ONCE = Sets.newHashSet();
    }
}
