package net.minecraft.world.entity.ai.attributes;

import java.util.Iterator;
import com.google.common.collect.Lists;
import javax.annotation.Nullable;
import java.util.Collection;
import com.google.common.collect.Sets;
import com.google.common.collect.Maps;
import java.util.UUID;
import java.util.Set;
import java.util.Map;

public class ModifiableAttributeInstance implements AttributeInstance {
    private final BaseAttributeMap attributeMap;
    private final Attribute attribute;
    private final Map<AttributeModifier.Operation, Set<AttributeModifier>> modifiers;
    private final Map<String, Set<AttributeModifier>> modifiersByName;
    private final Map<UUID, AttributeModifier> modifierById;
    private double baseValue;
    private boolean dirty;
    private double cachedValue;
    
    public ModifiableAttributeInstance(final BaseAttributeMap alr, final Attribute aln) {
        this.modifiers = Maps.newEnumMap(AttributeModifier.Operation.class);
        this.modifiersByName = Maps.newHashMap();
        this.modifierById = Maps.newHashMap();
        this.dirty = true;
        this.attributeMap = alr;
        this.attribute = aln;
        this.baseValue = aln.getDefaultValue();
        for (final AttributeModifier.Operation a7 : AttributeModifier.Operation.values()) {
            this.modifiers.put(a7, Sets.newHashSet());
        }
    }
    
    @Override
    public Attribute getAttribute() {
        return this.attribute;
    }
    
    @Override
    public double getBaseValue() {
        return this.baseValue;
    }
    
    @Override
    public void setBaseValue(final double double1) {
        if (double1 == this.getBaseValue()) {
            return;
        }
        this.baseValue = double1;
        this.setDirty();
    }
    
    @Override
    public Set<AttributeModifier> getModifiers(final AttributeModifier.Operation a) {
        return this.modifiers.get(a);
    }
    
    @Override
    public Set<AttributeModifier> getModifiers() {
        final Set<AttributeModifier> set2 = Sets.newHashSet();
        for (final AttributeModifier.Operation a6 : AttributeModifier.Operation.values()) {
            set2.addAll(this.getModifiers(a6));
        }
        return set2;
    }
    
    @Nullable
    @Override
    public AttributeModifier getModifier(final UUID uUID) {
        return this.modifierById.get(uUID);
    }
    
    @Override
    public boolean hasModifier(final AttributeModifier alp) {
        return this.modifierById.get(alp.getId()) != null;
    }
    
    @Override
    public void addModifier(final AttributeModifier alp) {
        if (this.getModifier(alp.getId()) != null) {
            throw new IllegalArgumentException("Modifier is already applied on this attribute!");
        }
        final Set<AttributeModifier> set3 = this.modifiersByName.computeIfAbsent(alp.getName(), string -> Sets.newHashSet());
        this.modifiers.get(alp.getOperation()).add(alp);
        set3.add(alp);
        this.modifierById.put(alp.getId(), alp);
        this.setDirty();
    }
    
    protected void setDirty() {
        this.dirty = true;
        this.attributeMap.onAttributeModified(this);
    }
    
    @Override
    public void removeModifier(final AttributeModifier alp) {
        for (final AttributeModifier.Operation a6 : AttributeModifier.Operation.values()) {
            this.modifiers.get(a6).remove(alp);
        }
        final Set<AttributeModifier> set3 = this.modifiersByName.get(alp.getName());
        if (set3 != null) {
            set3.remove(alp);
            if (set3.isEmpty()) {
                this.modifiersByName.remove(alp.getName());
            }
        }
        this.modifierById.remove(alp.getId());
        this.setDirty();
    }
    
    @Override
    public void removeModifier(final UUID uUID) {
        final AttributeModifier alp3 = this.getModifier(uUID);
        if (alp3 != null) {
            this.removeModifier(alp3);
        }
    }
    
    @Override
    public void removeModifiers() {
        Collection<AttributeModifier> collection2 = this.getModifiers();
        if (collection2 == null) {
            return;
        }
        collection2 = Lists.newArrayList(collection2);
        for (final AttributeModifier alp4 : collection2) {
            this.removeModifier(alp4);
        }
    }
    
    @Override
    public double getValue() {
        if (this.dirty) {
            this.cachedValue = this.calculateValue();
            this.dirty = false;
        }
        return this.cachedValue;
    }
    
    private double calculateValue() {
        double double2 = this.getBaseValue();
        for (final AttributeModifier alp5 : this.getAppliedModifiers(AttributeModifier.Operation.ADDITION)) {
            double2 += alp5.getAmount();
        }
        double double3 = double2;
        for (final AttributeModifier alp6 : this.getAppliedModifiers(AttributeModifier.Operation.MULTIPLY_BASE)) {
            double3 += double2 * alp6.getAmount();
        }
        for (final AttributeModifier alp6 : this.getAppliedModifiers(AttributeModifier.Operation.MULTIPLY_TOTAL)) {
            double3 *= 1.0 + alp6.getAmount();
        }
        return this.attribute.sanitizeValue(double3);
    }
    
    private Collection<AttributeModifier> getAppliedModifiers(final AttributeModifier.Operation a) {
        final Set<AttributeModifier> set3 = Sets.newHashSet(this.getModifiers(a));
        for (Attribute aln4 = this.attribute.getParentAttribute(); aln4 != null; aln4 = aln4.getParentAttribute()) {
            final AttributeInstance alo5 = this.attributeMap.getInstance(aln4);
            if (alo5 != null) {
                set3.addAll(alo5.getModifiers(a));
            }
        }
        return set3;
    }
}
