package net.minecraft.world.item.crafting;

import java.util.Iterator;
import java.util.List;
import com.google.common.collect.Lists;
import java.util.Collections;
import java.util.Collection;
import net.minecraft.tags.ItemTags;
import net.minecraft.core.Registry;
import net.minecraft.resources.ResourceLocation;
import com.google.gson.JsonParseException;
import com.google.gson.JsonObject;
import net.minecraft.util.GsonHelper;
import java.util.stream.StreamSupport;
import com.google.gson.JsonSyntaxException;
import net.minecraft.world.item.Item;
import net.minecraft.tags.Tag;
import net.minecraft.world.level.ItemLike;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import net.minecraft.network.FriendlyByteBuf;
import java.util.Comparator;
import it.unimi.dsi.fastutil.ints.IntComparators;
import net.minecraft.world.entity.player.StackedContents;
import it.unimi.dsi.fastutil.ints.IntArrayList;
import javax.annotation.Nullable;
import java.util.Arrays;
import java.util.stream.Stream;
import it.unimi.dsi.fastutil.ints.IntList;
import net.minecraft.world.item.ItemStack;
import java.util.function.Predicate;

public final class Ingredient implements Predicate<ItemStack> {
    private static final Predicate<? super Value> NON_ALL_EMPTY;
    public static final Ingredient EMPTY;
    private final Value[] values;
    private ItemStack[] itemStacks;
    private IntList stackingIds;
    
    private Ingredient(final Stream<? extends Value> stream) {
        this.values = stream.filter(Ingredient.NON_ALL_EMPTY).<Value>toArray(Value[]::new);
    }
    
    public ItemStack[] getItems() {
        this.dissolve();
        return this.itemStacks;
    }
    
    private void dissolve() {
        if (this.itemStacks == null) {
            this.itemStacks = Arrays.<Value>stream(this.values).flatMap(c -> c.getItems().stream()).distinct().<ItemStack>toArray(ItemStack[]::new);
        }
    }
    
    @Override
    public boolean test(@Nullable final ItemStack bek) {
        if (bek == null) {
            return false;
        }
        if (this.values.length == 0) {
            return bek.isEmpty();
        }
        this.dissolve();
        for (final ItemStack bek2 : this.itemStacks) {
            if (bek2.getItem() == bek.getItem()) {
                return true;
            }
        }
        return false;
    }
    
    public IntList getStackingIds() {
        if (this.stackingIds == null) {
            this.dissolve();
            this.stackingIds = (IntList)new IntArrayList(this.itemStacks.length);
            for (final ItemStack bek5 : this.itemStacks) {
                this.stackingIds.add(StackedContents.getStackingIndex(bek5));
            }
            this.stackingIds.sort((Comparator)IntComparators.NATURAL_COMPARATOR);
        }
        return this.stackingIds;
    }
    
    public void toNetwork(final FriendlyByteBuf kv) {
        this.dissolve();
        kv.writeVarInt(this.itemStacks.length);
        for (int integer3 = 0; integer3 < this.itemStacks.length; ++integer3) {
            kv.writeItem(this.itemStacks[integer3]);
        }
    }
    
    public JsonElement toJson() {
        if (this.values.length == 1) {
            return this.values[0].serialize();
        }
        final JsonArray jsonArray2 = new JsonArray();
        for (final Value c6 : this.values) {
            jsonArray2.add(c6.serialize());
        }
        return jsonArray2;
    }
    
    public boolean isEmpty() {
        return this.values.length == 0 && (this.itemStacks == null || this.itemStacks.length == 0) && (this.stackingIds == null || this.stackingIds.isEmpty());
    }
    
    private static Ingredient fromValues(final Stream<? extends Value> stream) {
        final Ingredient bgq2 = new Ingredient(stream);
        return (bgq2.values.length == 0) ? Ingredient.EMPTY : bgq2;
    }
    
    public static Ingredient of(final ItemLike... arr) {
        final ItemValue itemValue;
        return fromValues(Arrays.<ItemLike>stream(arr).map(bjs -> {
            new ItemValue(new ItemStack(bjs));
            return itemValue;
        }));
    }
    
    public static Ingredient of(final ItemStack... arr) {
        return fromValues(Arrays.<ItemStack>stream(arr).map(bek -> new ItemValue(bek)));
    }
    
    public static Ingredient of(final Tag<Item> aaz) {
        return fromValues(Stream.<TagValue>of(new TagValue((Tag)aaz)));
    }
    
    public static Ingredient fromNetwork(final FriendlyByteBuf kv) {
        final int integer2 = kv.readVarInt();
        return fromValues(Stream.generate(() -> new ItemValue(kv.readItem())).limit(integer2));
    }
    
    public static Ingredient fromJson(@Nullable final JsonElement jsonElement) {
        if (jsonElement == null || jsonElement.isJsonNull()) {
            throw new JsonSyntaxException("Item cannot be null");
        }
        if (jsonElement.isJsonObject()) {
            return fromValues(Stream.<Value>of(valueFromJson(jsonElement.getAsJsonObject())));
        }
        if (!jsonElement.isJsonArray()) {
            throw new JsonSyntaxException("Expected item to be object or array of objects");
        }
        final JsonArray jsonArray2 = jsonElement.getAsJsonArray();
        if (jsonArray2.size() == 0) {
            throw new JsonSyntaxException("Item array cannot be empty, at least one item must be defined");
        }
        return fromValues(StreamSupport.<JsonElement>stream(jsonArray2.spliterator(), false).map(jsonElement -> valueFromJson(GsonHelper.convertToJsonObject(jsonElement, "item"))));
    }
    
    public static Value valueFromJson(final JsonObject jsonObject) {
        if (jsonObject.has("item") && jsonObject.has("tag")) {
            throw new JsonParseException("An ingredient entry is either a tag or an item, not both");
        }
        if (jsonObject.has("item")) {
            final ResourceLocation sm2 = new ResourceLocation(GsonHelper.getAsString(jsonObject, "item"));
            final Object o;
            final Object obj;
            final Item bef3 = Registry.ITEM.getOptional(sm2).<Throwable>orElseThrow(() -> {
                new JsonSyntaxException("Unknown item '" + obj + "'");
                return o;
            });
            return new ItemValue(new ItemStack(bef3));
        }
        if (!jsonObject.has("tag")) {
            throw new JsonParseException("An ingredient entry needs either a tag or an item");
        }
        final ResourceLocation sm2 = new ResourceLocation(GsonHelper.getAsString(jsonObject, "tag"));
        final Tag<Item> aaz3 = ItemTags.getAllTags().getTag(sm2);
        if (aaz3 == null) {
            throw new JsonSyntaxException("Unknown item tag '" + sm2 + "'");
        }
        return new TagValue((Tag)aaz3);
    }
    
    static {
        NON_ALL_EMPTY = (c -> !c.getItems().stream().allMatch(ItemStack::isEmpty));
        EMPTY = new Ingredient(Stream.empty());
    }
    
    static class ItemValue implements Value {
        private final ItemStack item;
        
        private ItemValue(final ItemStack bek) {
            this.item = bek;
        }
        
        @Override
        public Collection<ItemStack> getItems() {
            return Collections.<ItemStack>singleton(this.item);
        }
        
        @Override
        public JsonObject serialize() {
            final JsonObject jsonObject2 = new JsonObject();
            jsonObject2.addProperty("item", Registry.ITEM.getKey(this.item.getItem()).toString());
            return jsonObject2;
        }
    }
    
    static class TagValue implements Value {
        private final Tag<Item> tag;
        
        private TagValue(final Tag<Item> aaz) {
            this.tag = aaz;
        }
        
        @Override
        public Collection<ItemStack> getItems() {
            final List<ItemStack> list2 = Lists.newArrayList();
            for (final Item bef4 : this.tag.getValues()) {
                list2.add(new ItemStack(bef4));
            }
            return list2;
        }
        
        @Override
        public JsonObject serialize() {
            final JsonObject jsonObject2 = new JsonObject();
            jsonObject2.addProperty("tag", this.tag.getId().toString());
            return jsonObject2;
        }
    }
    
    interface Value {
        Collection<ItemStack> getItems();
        
        JsonObject serialize();
    }
}
