package net.minecraft.world.level.block.entity;

import java.util.function.Supplier;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.level.block.BannerBlock;
import net.minecraft.world.level.block.state.BlockState;
import com.google.common.collect.Lists;
import net.minecraft.network.protocol.game.ClientboundBlockEntityDataPacket;
import net.minecraft.world.level.block.AbstractBannerBlock;
import net.minecraft.nbt.Tag;
import javax.annotation.Nullable;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.item.ItemStack;
import java.util.List;
import net.minecraft.nbt.ListTag;
import net.minecraft.world.item.DyeColor;
import net.minecraft.network.chat.Component;
import net.minecraft.world.Nameable;

public class BannerBlockEntity extends BlockEntity implements Nameable {
    private Component name;
    private DyeColor baseColor;
    private ListTag itemPatterns;
    private boolean receivedData;
    private List<BannerPattern> patterns;
    private List<DyeColor> colors;
    private String textureHashName;
    private boolean onlyRenderPattern;
    
    public BannerBlockEntity() {
        super(BlockEntityType.BANNER);
        this.baseColor = DyeColor.WHITE;
        this.onlyRenderPattern = false;
    }
    
    public BannerBlockEntity(final DyeColor bdg) {
        this();
        this.baseColor = bdg;
    }
    
    public void fromItem(final ItemStack bek, final DyeColor bdg) {
        this.itemPatterns = null;
        final CompoundTag jt4 = bek.getTagElement("BlockEntityTag");
        if (jt4 != null && jt4.contains("Patterns", 9)) {
            this.itemPatterns = jt4.getList("Patterns", 10).copy();
        }
        this.baseColor = bdg;
        this.patterns = null;
        this.colors = null;
        this.textureHashName = "";
        this.receivedData = true;
        this.name = (bek.hasCustomHoverName() ? bek.getHoverName() : null);
    }
    
    @Override
    public Component getName() {
        if (this.name != null) {
            return this.name;
        }
        return new TranslatableComponent("block.minecraft.banner", new Object[0]);
    }
    
    @Nullable
    @Override
    public Component getCustomName() {
        return this.name;
    }
    
    public void setCustomName(final Component lf) {
        this.name = lf;
    }
    
    @Override
    public CompoundTag save(final CompoundTag jt) {
        super.save(jt);
        if (this.itemPatterns != null) {
            jt.put("Patterns", this.itemPatterns);
        }
        if (this.name != null) {
            jt.putString("CustomName", Component.Serializer.toJson(this.name));
        }
        return jt;
    }
    
    @Override
    public void load(final CompoundTag jt) {
        super.load(jt);
        if (jt.contains("CustomName", 8)) {
            this.name = Component.Serializer.fromJson(jt.getString("CustomName"));
        }
        if (this.hasLevel()) {
            this.baseColor = ((AbstractBannerBlock)this.getBlockState().getBlock()).getColor();
        }
        else {
            this.baseColor = null;
        }
        this.itemPatterns = jt.getList("Patterns", 10);
        this.patterns = null;
        this.colors = null;
        this.textureHashName = null;
        this.receivedData = true;
    }
    
    @Nullable
    @Override
    public ClientboundBlockEntityDataPacket getUpdatePacket() {
        return new ClientboundBlockEntityDataPacket(this.worldPosition, 6, this.getUpdateTag());
    }
    
    @Override
    public CompoundTag getUpdateTag() {
        return this.save(new CompoundTag());
    }
    
    public static int getPatternCount(final ItemStack bek) {
        final CompoundTag jt2 = bek.getTagElement("BlockEntityTag");
        if (jt2 != null && jt2.contains("Patterns")) {
            return jt2.getList("Patterns", 10).size();
        }
        return 0;
    }
    
    public List<BannerPattern> getPatterns() {
        this.createPatternList();
        return this.patterns;
    }
    
    public List<DyeColor> getColors() {
        this.createPatternList();
        return this.colors;
    }
    
    private void createPatternList() {
        if (this.patterns != null && this.colors != null && this.textureHashName != null) {
            return;
        }
        if (!this.receivedData) {
            this.textureHashName = "";
            return;
        }
        this.patterns = Lists.newArrayList();
        this.colors = Lists.newArrayList();
        final DyeColor bdg2 = this.getBaseColor(this::getBlockState);
        if (bdg2 == null) {
            this.textureHashName = "banner_missing";
        }
        else {
            this.patterns.add(BannerPattern.BASE);
            this.colors.add(bdg2);
            this.textureHashName = "b" + bdg2.getId();
            if (this.itemPatterns != null) {
                for (int integer3 = 0; integer3 < this.itemPatterns.size(); ++integer3) {
                    final CompoundTag jt4 = this.itemPatterns.getCompound(integer3);
                    final BannerPattern bwa5 = BannerPattern.byHash(jt4.getString("Pattern"));
                    if (bwa5 != null) {
                        this.patterns.add(bwa5);
                        final int integer4 = jt4.getInt("Color");
                        this.colors.add(DyeColor.byId(integer4));
                        this.textureHashName = this.textureHashName + bwa5.getHashname() + integer4;
                    }
                }
            }
        }
    }
    
    public static void removeLastPattern(final ItemStack bek) {
        final CompoundTag jt2 = bek.getTagElement("BlockEntityTag");
        if (jt2 == null || !jt2.contains("Patterns", 9)) {
            return;
        }
        final ListTag jz3 = jt2.getList("Patterns", 10);
        if (jz3.isEmpty()) {
            return;
        }
        jz3.remove(jz3.size() - 1);
        if (jz3.isEmpty()) {
            bek.removeTagKey("BlockEntityTag");
        }
    }
    
    public ItemStack getItem(final BlockState byg) {
        final ItemStack bek3 = new ItemStack(BannerBlock.byColor(this.getBaseColor(() -> byg)));
        if (this.itemPatterns != null && !this.itemPatterns.isEmpty()) {
            bek3.getOrCreateTagElement("BlockEntityTag").put("Patterns", this.itemPatterns.copy());
        }
        if (this.name != null) {
            bek3.setHoverName(this.name);
        }
        return bek3;
    }
    
    public DyeColor getBaseColor(final Supplier<BlockState> supplier) {
        if (this.baseColor == null) {
            this.baseColor = ((AbstractBannerBlock)supplier.get().getBlock()).getColor();
        }
        return this.baseColor;
    }
    
    public void setOnlyRenderPattern(final boolean boolean1) {
        this.onlyRenderPattern = boolean1;
    }
    
    public boolean onlyRenderPattern() {
        return this.onlyRenderPattern;
    }
}
