package net.minecraft.world.level.levelgen.structure;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import com.google.common.collect.ImmutableSet;
import net.minecraft.world.level.block.entity.DispenserBlockEntity;
import net.minecraft.world.level.block.DispenserBlock;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.ChestBlockEntity;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.HorizontalDirectionalBlock;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.level.levelgen.Heightmap;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.core.Vec3i;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import java.util.Iterator;
import net.minecraft.world.level.ChunkPos;
import net.minecraft.world.level.chunk.ChunkGenerator;
import net.minecraft.world.level.LevelAccessor;
import java.util.Random;
import java.util.List;
import net.minecraft.nbt.Tag;
import net.minecraft.core.Registry;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.block.Block;
import java.util.Set;
import net.minecraft.world.level.levelgen.feature.StructurePieceType;
import net.minecraft.world.level.block.Rotation;
import net.minecraft.world.level.block.Mirror;
import javax.annotation.Nullable;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.state.BlockState;

public abstract class StructurePiece {
    protected static final BlockState CAVE_AIR;
    protected BoundingBox boundingBox;
    @Nullable
    private Direction orientation;
    private Mirror mirror;
    private Rotation rotation;
    protected int genDepth;
    private final StructurePieceType type;
    private static final Set<Block> SHAPE_CHECK_BLOCKS;
    
    protected StructurePiece(final StructurePieceType cfj, final int integer) {
        this.type = cfj;
        this.genDepth = integer;
    }
    
    public StructurePiece(final StructurePieceType cfj, final CompoundTag jt) {
        this(cfj, jt.getInt("GD"));
        if (jt.contains("BB")) {
            this.boundingBox = new BoundingBox(jt.getIntArray("BB"));
        }
        final int integer4 = jt.getInt("O");
        this.setOrientation((integer4 == -1) ? null : Direction.from2DDataValue(integer4));
    }
    
    public final CompoundTag createTag() {
        final CompoundTag jt2 = new CompoundTag();
        jt2.putString("id", Registry.STRUCTURE_PIECE.getKey(this.getType()).toString());
        jt2.put("BB", this.boundingBox.createTag());
        final Direction fp3 = this.getOrientation();
        jt2.putInt("O", (fp3 == null) ? -1 : fp3.get2DDataValue());
        jt2.putInt("GD", this.genDepth);
        this.addAdditionalSaveData(jt2);
        return jt2;
    }
    
    protected abstract void addAdditionalSaveData(final CompoundTag jt);
    
    public void addChildren(final StructurePiece clr, final List<StructurePiece> list, final Random random) {
    }
    
    public abstract boolean postProcess(final LevelAccessor bju, final ChunkGenerator<?> bzx, final Random random, final BoundingBox cky, final ChunkPos bje);
    
    public BoundingBox getBoundingBox() {
        return this.boundingBox;
    }
    
    public int getGenDepth() {
        return this.genDepth;
    }
    
    public boolean isCloseToChunk(final ChunkPos bje, final int integer) {
        final int integer2 = bje.x << 4;
        final int integer3 = bje.z << 4;
        return this.boundingBox.intersects(integer2 - integer, integer3 - integer, integer2 + 15 + integer, integer3 + 15 + integer);
    }
    
    public static StructurePiece findCollisionPiece(final List<StructurePiece> list, final BoundingBox cky) {
        for (final StructurePiece clr4 : list) {
            if (clr4.getBoundingBox() != null && clr4.getBoundingBox().intersects(cky)) {
                return clr4;
            }
        }
        return null;
    }
    
    protected boolean edgesLiquid(final BlockGetter bjd, final BoundingBox cky) {
        final int integer4 = Math.max(this.boundingBox.x0 - 1, cky.x0);
        final int integer5 = Math.max(this.boundingBox.y0 - 1, cky.y0);
        final int integer6 = Math.max(this.boundingBox.z0 - 1, cky.z0);
        final int integer7 = Math.min(this.boundingBox.x1 + 1, cky.x1);
        final int integer8 = Math.min(this.boundingBox.y1 + 1, cky.y1);
        final int integer9 = Math.min(this.boundingBox.z1 + 1, cky.z1);
        final BlockPos.MutableBlockPos a10 = new BlockPos.MutableBlockPos();
        for (int integer10 = integer4; integer10 <= integer7; ++integer10) {
            for (int integer11 = integer6; integer11 <= integer9; ++integer11) {
                if (bjd.getBlockState(a10.set(integer10, integer5, integer11)).getMaterial().isLiquid()) {
                    return true;
                }
                if (bjd.getBlockState(a10.set(integer10, integer8, integer11)).getMaterial().isLiquid()) {
                    return true;
                }
            }
        }
        for (int integer10 = integer4; integer10 <= integer7; ++integer10) {
            for (int integer11 = integer5; integer11 <= integer8; ++integer11) {
                if (bjd.getBlockState(a10.set(integer10, integer11, integer6)).getMaterial().isLiquid()) {
                    return true;
                }
                if (bjd.getBlockState(a10.set(integer10, integer11, integer9)).getMaterial().isLiquid()) {
                    return true;
                }
            }
        }
        for (int integer10 = integer6; integer10 <= integer9; ++integer10) {
            for (int integer11 = integer5; integer11 <= integer8; ++integer11) {
                if (bjd.getBlockState(a10.set(integer4, integer11, integer10)).getMaterial().isLiquid()) {
                    return true;
                }
                if (bjd.getBlockState(a10.set(integer7, integer11, integer10)).getMaterial().isLiquid()) {
                    return true;
                }
            }
        }
        return false;
    }
    
    protected int getWorldX(final int integer1, final int integer2) {
        final Direction fp4 = this.getOrientation();
        if (fp4 == null) {
            return integer1;
        }
        switch (fp4) {
            case NORTH:
            case SOUTH: {
                return this.boundingBox.x0 + integer1;
            }
            case WEST: {
                return this.boundingBox.x1 - integer2;
            }
            case EAST: {
                return this.boundingBox.x0 + integer2;
            }
            default: {
                return integer1;
            }
        }
    }
    
    protected int getWorldY(final int integer) {
        if (this.getOrientation() == null) {
            return integer;
        }
        return integer + this.boundingBox.y0;
    }
    
    protected int getWorldZ(final int integer1, final int integer2) {
        final Direction fp4 = this.getOrientation();
        if (fp4 == null) {
            return integer2;
        }
        switch (fp4) {
            case NORTH: {
                return this.boundingBox.z1 - integer2;
            }
            case SOUTH: {
                return this.boundingBox.z0 + integer2;
            }
            case WEST:
            case EAST: {
                return this.boundingBox.z0 + integer1;
            }
            default: {
                return integer2;
            }
        }
    }
    
    protected void placeBlock(final LevelAccessor bju, BlockState byg, final int integer3, final int integer4, final int integer5, final BoundingBox cky) {
        final BlockPos fk8 = new BlockPos(this.getWorldX(integer3, integer5), this.getWorldY(integer4), this.getWorldZ(integer3, integer5));
        if (!cky.isInside(fk8)) {
            return;
        }
        if (this.mirror != Mirror.NONE) {
            byg = byg.mirror(this.mirror);
        }
        if (this.rotation != Rotation.NONE) {
            byg = byg.rotate(this.rotation);
        }
        bju.setBlock(fk8, byg, 2);
        final FluidState cog9 = bju.getFluidState(fk8);
        if (!cog9.isEmpty()) {
            bju.getLiquidTicks().scheduleTick(fk8, cog9.getType(), 0);
        }
        if (StructurePiece.SHAPE_CHECK_BLOCKS.contains(byg.getBlock())) {
            bju.getChunk(fk8).markPosForPostprocessing(fk8);
        }
    }
    
    protected BlockState getBlock(final BlockGetter bjd, final int integer2, final int integer3, final int integer4, final BoundingBox cky) {
        final int integer5 = this.getWorldX(integer2, integer4);
        final int integer6 = this.getWorldY(integer3);
        final int integer7 = this.getWorldZ(integer2, integer4);
        final BlockPos fk10 = new BlockPos(integer5, integer6, integer7);
        if (!cky.isInside(fk10)) {
            return Blocks.AIR.defaultBlockState();
        }
        return bjd.getBlockState(fk10);
    }
    
    protected boolean isInterior(final LevelReader bjw, final int integer2, final int integer3, final int integer4, final BoundingBox cky) {
        final int integer5 = this.getWorldX(integer2, integer4);
        final int integer6 = this.getWorldY(integer3 + 1);
        final int integer7 = this.getWorldZ(integer2, integer4);
        final BlockPos fk10 = new BlockPos(integer5, integer6, integer7);
        return cky.isInside(fk10) && integer6 < bjw.getHeight(Heightmap.Types.OCEAN_FLOOR_WG, integer5, integer7);
    }
    
    protected void generateAirBox(final LevelAccessor bju, final BoundingBox cky, final int integer3, final int integer4, final int integer5, final int integer6, final int integer7, final int integer8) {
        for (int integer9 = integer4; integer9 <= integer7; ++integer9) {
            for (int integer10 = integer3; integer10 <= integer6; ++integer10) {
                for (int integer11 = integer5; integer11 <= integer8; ++integer11) {
                    this.placeBlock(bju, Blocks.AIR.defaultBlockState(), integer10, integer9, integer11, cky);
                }
            }
        }
    }
    
    protected void generateBox(final LevelAccessor bju, final BoundingBox cky, final int integer3, final int integer4, final int integer5, final int integer6, final int integer7, final int integer8, final BlockState byg9, final BlockState byg10, final boolean boolean11) {
        for (int integer9 = integer4; integer9 <= integer7; ++integer9) {
            for (int integer10 = integer3; integer10 <= integer6; ++integer10) {
                for (int integer11 = integer5; integer11 <= integer8; ++integer11) {
                    if (!boolean11 || !this.getBlock(bju, integer10, integer9, integer11, cky).isAir()) {
                        if (integer9 == integer4 || integer9 == integer7 || integer10 == integer3 || integer10 == integer6 || integer11 == integer5 || integer11 == integer8) {
                            this.placeBlock(bju, byg9, integer10, integer9, integer11, cky);
                        }
                        else {
                            this.placeBlock(bju, byg10, integer10, integer9, integer11, cky);
                        }
                    }
                }
            }
        }
    }
    
    protected void generateBox(final LevelAccessor bju, final BoundingBox cky, final int integer3, final int integer4, final int integer5, final int integer6, final int integer7, final int integer8, final boolean boolean9, final Random random, final BlockSelector a) {
        for (int integer9 = integer4; integer9 <= integer7; ++integer9) {
            for (int integer10 = integer3; integer10 <= integer6; ++integer10) {
                for (int integer11 = integer5; integer11 <= integer8; ++integer11) {
                    if (!boolean9 || !this.getBlock(bju, integer10, integer9, integer11, cky).isAir()) {
                        a.next(random, integer10, integer9, integer11, integer9 == integer4 || integer9 == integer7 || integer10 == integer3 || integer10 == integer6 || integer11 == integer5 || integer11 == integer8);
                        this.placeBlock(bju, a.getNext(), integer10, integer9, integer11, cky);
                    }
                }
            }
        }
    }
    
    protected void generateMaybeBox(final LevelAccessor bju, final BoundingBox cky, final Random random, final float float4, final int integer5, final int integer6, final int integer7, final int integer8, final int integer9, final int integer10, final BlockState byg11, final BlockState byg12, final boolean boolean13, final boolean boolean14) {
        for (int integer11 = integer6; integer11 <= integer9; ++integer11) {
            for (int integer12 = integer5; integer12 <= integer8; ++integer12) {
                for (int integer13 = integer7; integer13 <= integer10; ++integer13) {
                    if (random.nextFloat() <= float4) {
                        if (!boolean13 || !this.getBlock(bju, integer12, integer11, integer13, cky).isAir()) {
                            if (!boolean14 || this.isInterior(bju, integer12, integer11, integer13, cky)) {
                                if (integer11 == integer6 || integer11 == integer9 || integer12 == integer5 || integer12 == integer8 || integer13 == integer7 || integer13 == integer10) {
                                    this.placeBlock(bju, byg11, integer12, integer11, integer13, cky);
                                }
                                else {
                                    this.placeBlock(bju, byg12, integer12, integer11, integer13, cky);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    
    protected void maybeGenerateBlock(final LevelAccessor bju, final BoundingBox cky, final Random random, final float float4, final int integer5, final int integer6, final int integer7, final BlockState byg) {
        if (random.nextFloat() < float4) {
            this.placeBlock(bju, byg, integer5, integer6, integer7, cky);
        }
    }
    
    protected void generateUpperHalfSphere(final LevelAccessor bju, final BoundingBox cky, final int integer3, final int integer4, final int integer5, final int integer6, final int integer7, final int integer8, final BlockState byg, final boolean boolean10) {
        final float float12 = (float)(integer6 - integer3 + 1);
        final float float13 = (float)(integer7 - integer4 + 1);
        final float float14 = (float)(integer8 - integer5 + 1);
        final float float15 = integer3 + float12 / 2.0f;
        final float float16 = integer5 + float14 / 2.0f;
        for (int integer9 = integer4; integer9 <= integer7; ++integer9) {
            final float float17 = (integer9 - integer4) / float13;
            for (int integer10 = integer3; integer10 <= integer6; ++integer10) {
                final float float18 = (integer10 - float15) / (float12 * 0.5f);
                for (int integer11 = integer5; integer11 <= integer8; ++integer11) {
                    final float float19 = (integer11 - float16) / (float14 * 0.5f);
                    if (!boolean10 || !this.getBlock(bju, integer10, integer9, integer11, cky).isAir()) {
                        final float float20 = float18 * float18 + float17 * float17 + float19 * float19;
                        if (float20 <= 1.05f) {
                            this.placeBlock(bju, byg, integer10, integer9, integer11, cky);
                        }
                    }
                }
            }
        }
    }
    
    protected void fillColumnDown(final LevelAccessor bju, final BlockState byg, final int integer3, final int integer4, final int integer5, final BoundingBox cky) {
        final int integer6 = this.getWorldX(integer3, integer5);
        int integer7 = this.getWorldY(integer4);
        final int integer8 = this.getWorldZ(integer3, integer5);
        if (!cky.isInside(new BlockPos(integer6, integer7, integer8))) {
            return;
        }
        while ((bju.isEmptyBlock(new BlockPos(integer6, integer7, integer8)) || bju.getBlockState(new BlockPos(integer6, integer7, integer8)).getMaterial().isLiquid()) && integer7 > 1) {
            bju.setBlock(new BlockPos(integer6, integer7, integer8), byg, 2);
            --integer7;
        }
    }
    
    protected boolean createChest(final LevelAccessor bju, final BoundingBox cky, final Random random, final int integer4, final int integer5, final int integer6, final ResourceLocation sm) {
        final BlockPos fk9 = new BlockPos(this.getWorldX(integer4, integer6), this.getWorldY(integer5), this.getWorldZ(integer4, integer6));
        return this.createChest(bju, cky, random, fk9, sm, null);
    }
    
    public static BlockState reorient(final BlockGetter bjd, final BlockPos fk, final BlockState byg) {
        Direction fp4 = null;
        for (final Direction fp5 : Direction.Plane.HORIZONTAL) {
            final BlockPos fk2 = fk.relative(fp5);
            final BlockState byg2 = bjd.getBlockState(fk2);
            if (byg2.getBlock() == Blocks.CHEST) {
                return byg;
            }
            if (!byg2.isSolidRender(bjd, fk2)) {
                continue;
            }
            if (fp4 != null) {
                fp4 = null;
                break;
            }
            fp4 = fp5;
        }
        if (fp4 != null) {
            return ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Direction>setValue((Property<Comparable>)HorizontalDirectionalBlock.FACING, fp4.getOpposite());
        }
        Direction fp6 = byg.<Direction>getValue((Property<Direction>)HorizontalDirectionalBlock.FACING);
        BlockPos fk3 = fk.relative(fp6);
        if (bjd.getBlockState(fk3).isSolidRender(bjd, fk3)) {
            fp6 = fp6.getOpposite();
            fk3 = fk.relative(fp6);
        }
        if (bjd.getBlockState(fk3).isSolidRender(bjd, fk3)) {
            fp6 = fp6.getClockWise();
            fk3 = fk.relative(fp6);
        }
        if (bjd.getBlockState(fk3).isSolidRender(bjd, fk3)) {
            fp6 = fp6.getOpposite();
            fk3 = fk.relative(fp6);
        }
        return ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Direction>setValue((Property<Comparable>)HorizontalDirectionalBlock.FACING, fp6);
    }
    
    protected boolean createChest(final LevelAccessor bju, final BoundingBox cky, final Random random, final BlockPos fk, final ResourceLocation sm, @Nullable BlockState byg) {
        if (!cky.isInside(fk) || bju.getBlockState(fk).getBlock() == Blocks.CHEST) {
            return false;
        }
        if (byg == null) {
            byg = reorient(bju, fk, Blocks.CHEST.defaultBlockState());
        }
        bju.setBlock(fk, byg, 2);
        final BlockEntity bwi8 = bju.getBlockEntity(fk);
        if (bwi8 instanceof ChestBlockEntity) {
            ((ChestBlockEntity)bwi8).setLootTable(sm, random.nextLong());
        }
        return true;
    }
    
    protected boolean createDispenser(final LevelAccessor bju, final BoundingBox cky, final Random random, final int integer4, final int integer5, final int integer6, final Direction fp, final ResourceLocation sm) {
        final BlockPos fk10 = new BlockPos(this.getWorldX(integer4, integer6), this.getWorldY(integer5), this.getWorldZ(integer4, integer6));
        if (cky.isInside(fk10) && bju.getBlockState(fk10).getBlock() != Blocks.DISPENSER) {
            this.placeBlock(bju, ((AbstractStateHolder<O, BlockState>)Blocks.DISPENSER.defaultBlockState()).<Comparable, Direction>setValue((Property<Comparable>)DispenserBlock.FACING, fp), integer4, integer5, integer6, cky);
            final BlockEntity bwi11 = bju.getBlockEntity(fk10);
            if (bwi11 instanceof DispenserBlockEntity) {
                ((DispenserBlockEntity)bwi11).setLootTable(sm, random.nextLong());
            }
            return true;
        }
        return false;
    }
    
    public void move(final int integer1, final int integer2, final int integer3) {
        this.boundingBox.move(integer1, integer2, integer3);
    }
    
    @Nullable
    public Direction getOrientation() {
        return this.orientation;
    }
    
    public void setOrientation(@Nullable final Direction fp) {
        this.orientation = fp;
        if (fp == null) {
            this.rotation = Rotation.NONE;
            this.mirror = Mirror.NONE;
        }
        else {
            switch (fp) {
                case SOUTH: {
                    this.mirror = Mirror.LEFT_RIGHT;
                    this.rotation = Rotation.NONE;
                    break;
                }
                case WEST: {
                    this.mirror = Mirror.LEFT_RIGHT;
                    this.rotation = Rotation.CLOCKWISE_90;
                    break;
                }
                case EAST: {
                    this.mirror = Mirror.NONE;
                    this.rotation = Rotation.CLOCKWISE_90;
                    break;
                }
                default: {
                    this.mirror = Mirror.NONE;
                    this.rotation = Rotation.NONE;
                    break;
                }
            }
        }
    }
    
    public Rotation getRotation() {
        return this.rotation;
    }
    
    public StructurePieceType getType() {
        return this.type;
    }
    
    static {
        CAVE_AIR = Blocks.CAVE_AIR.defaultBlockState();
        SHAPE_CHECK_BLOCKS = ImmutableSet.<Block>builder().add(Blocks.NETHER_BRICK_FENCE).add(Blocks.TORCH).add(Blocks.WALL_TORCH).add(Blocks.OAK_FENCE).add(Blocks.SPRUCE_FENCE).add(Blocks.DARK_OAK_FENCE).add(Blocks.ACACIA_FENCE).add(Blocks.BIRCH_FENCE).add(Blocks.JUNGLE_FENCE).add(Blocks.LADDER).add(Blocks.IRON_BARS).build();
    }
    
    public abstract static class BlockSelector {
        protected BlockState next;
        
        protected BlockSelector() {
            this.next = Blocks.AIR.defaultBlockState();
        }
        
        public abstract void next(final Random random, final int integer2, final int integer3, final int integer4, final boolean boolean5);
        
        public BlockState getNext() {
            return this.next;
        }
    }
}
