package net.minecraft.world.level.lighting;

import net.minecraft.world.level.chunk.DataLayer;
import net.minecraft.world.level.LightLayer;
import net.minecraft.world.level.ChunkPos;
import net.minecraft.core.SectionPos;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.chunk.LightChunkGetter;
import javax.annotation.Nullable;

public class LevelLightEngine implements LightEventListener {
    @Nullable
    private final LayerLightEngine<?, ?> blockEngine;
    @Nullable
    private final LayerLightEngine<?, ?> skyEngine;
    
    public LevelLightEngine(final LightChunkGetter cak, final boolean boolean2, final boolean boolean3) {
        this.blockEngine = (boolean2 ? new BlockLightEngine(cak) : null);
        this.skyEngine = (boolean3 ? new SkyLightEngine(cak) : null);
    }
    
    public void checkBlock(final BlockPos fk) {
        if (this.blockEngine != null) {
            this.blockEngine.checkBlock(fk);
        }
        if (this.skyEngine != null) {
            this.skyEngine.checkBlock(fk);
        }
    }
    
    public void onBlockEmissionIncrease(final BlockPos fk, final int integer) {
        if (this.blockEngine != null) {
            this.blockEngine.onBlockEmissionIncrease(fk, integer);
        }
    }
    
    public boolean hasLightWork() {
        return (this.skyEngine != null && this.skyEngine.hasLightWork()) || (this.blockEngine != null && this.blockEngine.hasLightWork());
    }
    
    public int runUpdates(final int integer, final boolean boolean2, final boolean boolean3) {
        if (this.blockEngine != null && this.skyEngine != null) {
            final int integer2 = integer / 2;
            final int integer3 = this.blockEngine.runUpdates(integer2, boolean2, boolean3);
            final int integer4 = integer - integer2 + integer3;
            final int integer5 = this.skyEngine.runUpdates(integer4, boolean2, boolean3);
            if (integer3 == 0 && integer5 > 0) {
                return this.blockEngine.runUpdates(integer5, boolean2, boolean3);
            }
            return integer5;
        }
        else {
            if (this.blockEngine != null) {
                return this.blockEngine.runUpdates(integer, boolean2, boolean3);
            }
            if (this.skyEngine != null) {
                return this.skyEngine.runUpdates(integer, boolean2, boolean3);
            }
            return integer;
        }
    }
    
    @Override
    public void updateSectionStatus(final SectionPos gd, final boolean boolean2) {
        if (this.blockEngine != null) {
            this.blockEngine.updateSectionStatus(gd, boolean2);
        }
        if (this.skyEngine != null) {
            this.skyEngine.updateSectionStatus(gd, boolean2);
        }
    }
    
    public void enableLightSources(final ChunkPos bje, final boolean boolean2) {
        if (this.blockEngine != null) {
            this.blockEngine.enableLightSources(bje, boolean2);
        }
        if (this.skyEngine != null) {
            this.skyEngine.enableLightSources(bje, boolean2);
        }
    }
    
    public LayerLightEventListener getLayerListener(final LightLayer bkc) {
        if (bkc == LightLayer.BLOCK) {
            if (this.blockEngine == null) {
                return LayerLightEventListener.DummyLightLayerEventListener.INSTANCE;
            }
            return this.blockEngine;
        }
        else {
            if (this.skyEngine == null) {
                return LayerLightEventListener.DummyLightLayerEventListener.INSTANCE;
            }
            return this.skyEngine;
        }
    }
    
    public String getDebugData(final LightLayer bkc, final SectionPos gd) {
        if (bkc == LightLayer.BLOCK) {
            if (this.blockEngine != null) {
                return this.blockEngine.getDebugData(gd.asLong());
            }
        }
        else if (this.skyEngine != null) {
            return this.skyEngine.getDebugData(gd.asLong());
        }
        return "n/a";
    }
    
    public void queueSectionData(final LightLayer bkc, final SectionPos gd, @Nullable final DataLayer cac) {
        if (bkc == LightLayer.BLOCK) {
            if (this.blockEngine != null) {
                this.blockEngine.queueSectionData(gd.asLong(), cac);
            }
        }
        else if (this.skyEngine != null) {
            this.skyEngine.queueSectionData(gd.asLong(), cac);
        }
    }
    
    public void retainData(final ChunkPos bje, final boolean boolean2) {
        if (this.blockEngine != null) {
            this.blockEngine.retainData(bje, boolean2);
        }
        if (this.skyEngine != null) {
            this.skyEngine.retainData(bje, boolean2);
        }
    }
    
    public int getRawBrightness(final BlockPos fk, final int integer) {
        final int integer2 = (this.skyEngine == null) ? 0 : (this.skyEngine.getLightValue(fk) - integer);
        final int integer3 = (this.blockEngine == null) ? 0 : this.blockEngine.getLightValue(fk);
        return Math.max(integer3, integer2);
    }
}
