package net.minecraft.world.level.storage.loot;

import org.apache.commons.lang3.ArrayUtils;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonParseException;
import com.google.gson.JsonObject;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.GsonHelper;
import com.google.gson.JsonDeserializationContext;
import java.lang.reflect.Type;
import com.google.gson.JsonElement;
import com.google.gson.JsonSerializer;
import com.google.gson.JsonDeserializer;
import net.minecraft.world.level.storage.loot.functions.FunctionUserBuilder;
import net.minecraft.world.level.storage.loot.parameters.LootContextParamSets;
import org.apache.logging.log4j.LogManager;
import java.util.Collections;
import java.util.Collection;
import net.minecraft.util.Mth;
import java.util.Iterator;
import java.util.Random;
import net.minecraft.world.Container;
import com.google.common.collect.Lists;
import java.util.List;
import java.util.function.Consumer;
import net.minecraft.world.level.storage.loot.functions.LootItemFunctions;
import net.minecraft.world.item.ItemStack;
import java.util.function.BiFunction;
import net.minecraft.world.level.storage.loot.functions.LootItemFunction;
import net.minecraft.world.level.storage.loot.parameters.LootContextParamSet;
import org.apache.logging.log4j.Logger;

public class LootTable {
    private static final Logger LOGGER;
    public static final LootTable EMPTY;
    public static final LootContextParamSet DEFAULT_PARAM_SET;
    private final LootContextParamSet paramSet;
    private final LootPool[] pools;
    private final LootItemFunction[] functions;
    private final BiFunction<ItemStack, LootContext, ItemStack> compositeFunction;
    
    private LootTable(final LootContextParamSet ctv, final LootPool[] arr, final LootItemFunction[] arr) {
        this.paramSet = ctv;
        this.pools = arr;
        this.functions = arr;
        this.compositeFunction = LootItemFunctions.compose(arr);
    }
    
    public static Consumer<ItemStack> createStackSplitter(final Consumer<ItemStack> consumer) {
        int integer3;
        ItemStack bek2;
        return bek -> {
            if (bek.getCount() < bek.getMaxStackSize()) {
                consumer.accept(bek);
            }
            else {
                integer3 = bek.getCount();
                while (integer3 > 0) {
                    bek2 = bek.copy();
                    bek2.setCount(Math.min(bek.getMaxStackSize(), integer3));
                    integer3 -= bek2.getCount();
                    consumer.accept(bek2);
                }
            }
        };
    }
    
    public void getRandomItemsRaw(final LootContext cru, final Consumer<ItemStack> consumer) {
        if (cru.addVisitedTable(this)) {
            final Consumer<ItemStack> consumer2 = LootItemFunction.decorate(this.compositeFunction, consumer, cru);
            for (final LootPool crw8 : this.pools) {
                crw8.addRandomItems(consumer2, cru);
            }
            cru.removeVisitedTable(this);
        }
        else {
            LootTable.LOGGER.warn("Detected infinite loop in loot tables");
        }
    }
    
    public void getRandomItems(final LootContext cru, final Consumer<ItemStack> consumer) {
        this.getRandomItemsRaw(cru, createStackSplitter(consumer));
    }
    
    public List<ItemStack> getRandomItems(final LootContext cru) {
        final List<ItemStack> list3 = Lists.newArrayList();
        this.getRandomItems(cru, list3::add);
        return list3;
    }
    
    public LootContextParamSet getParamSet() {
        return this.paramSet;
    }
    
    public void validate(final ValidationContext csd) {
        for (int integer3 = 0; integer3 < this.pools.length; ++integer3) {
            this.pools[integer3].validate(csd.forChild(".pools[" + integer3 + "]"));
        }
        for (int integer3 = 0; integer3 < this.functions.length; ++integer3) {
            this.functions[integer3].validate(csd.forChild(".functions[" + integer3 + "]"));
        }
    }
    
    public void fill(final Container ajb, final LootContext cru) {
        final List<ItemStack> list4 = this.getRandomItems(cru);
        final Random random5 = cru.getRandom();
        final List<Integer> list5 = this.getAvailableSlots(ajb, random5);
        this.shuffleAndSplitItems(list4, list5.size(), random5);
        for (final ItemStack bek8 : list4) {
            if (list5.isEmpty()) {
                LootTable.LOGGER.warn("Tried to over-fill a container");
                return;
            }
            if (bek8.isEmpty()) {
                ajb.setItem(list5.remove(list5.size() - 1), ItemStack.EMPTY);
            }
            else {
                ajb.setItem(list5.remove(list5.size() - 1), bek8);
            }
        }
    }
    
    private void shuffleAndSplitItems(final List<ItemStack> list, final int integer, final Random random) {
        final List<ItemStack> list2 = Lists.newArrayList();
        final Iterator<ItemStack> iterator6 = list.iterator();
        while (iterator6.hasNext()) {
            final ItemStack bek7 = iterator6.next();
            if (bek7.isEmpty()) {
                iterator6.remove();
            }
            else {
                if (bek7.getCount() <= 1) {
                    continue;
                }
                list2.add(bek7);
                iterator6.remove();
            }
        }
        while (integer - list.size() - list2.size() > 0 && !list2.isEmpty()) {
            final ItemStack bek8 = list2.remove(Mth.nextInt(random, 0, list2.size() - 1));
            final int integer2 = Mth.nextInt(random, 1, bek8.getCount() / 2);
            final ItemStack bek9 = bek8.split(integer2);
            if (bek8.getCount() > 1 && random.nextBoolean()) {
                list2.add(bek8);
            }
            else {
                list.add(bek8);
            }
            if (bek9.getCount() > 1 && random.nextBoolean()) {
                list2.add(bek9);
            }
            else {
                list.add(bek9);
            }
        }
        list.addAll(list2);
        Collections.shuffle(list, random);
    }
    
    private List<Integer> getAvailableSlots(final Container ajb, final Random random) {
        final List<Integer> list4 = Lists.newArrayList();
        for (int integer5 = 0; integer5 < ajb.getContainerSize(); ++integer5) {
            if (ajb.getItem(integer5).isEmpty()) {
                list4.add(integer5);
            }
        }
        Collections.shuffle(list4, random);
        return list4;
    }
    
    public static Builder lootTable() {
        return new Builder();
    }
    
    static {
        LOGGER = LogManager.getLogger();
        EMPTY = new LootTable(LootContextParamSets.EMPTY, new LootPool[0], new LootItemFunction[0]);
        DEFAULT_PARAM_SET = LootContextParamSets.ALL_PARAMS;
    }
    
    public static class Builder implements FunctionUserBuilder<Builder> {
        private final List<LootPool> pools;
        private final List<LootItemFunction> functions;
        private LootContextParamSet paramSet;
        
        public Builder() {
            this.pools = Lists.newArrayList();
            this.functions = Lists.newArrayList();
            this.paramSet = LootTable.DEFAULT_PARAM_SET;
        }
        
        public Builder withPool(final LootPool.Builder a) {
            this.pools.add(a.build());
            return this;
        }
        
        public Builder setParamSet(final LootContextParamSet ctv) {
            this.paramSet = ctv;
            return this;
        }
        
        @Override
        public Builder apply(final LootItemFunction.Builder a) {
            this.functions.add(a.build());
            return this;
        }
        
        @Override
        public Builder unwrap() {
            return this;
        }
        
        public LootTable build() {
            return new LootTable(this.paramSet, this.pools.<LootPool>toArray(new LootPool[0]), this.functions.<LootItemFunction>toArray(new LootItemFunction[0]), null);
        }
    }
    
    public static class Serializer implements JsonDeserializer<LootTable>, JsonSerializer<LootTable> {
        @Override
        public LootTable deserialize(final JsonElement jsonElement, final Type type, final JsonDeserializationContext jsonDeserializationContext) throws JsonParseException {
            final JsonObject jsonObject5 = GsonHelper.convertToJsonObject(jsonElement, "loot table");
            final LootPool[] arr6 = GsonHelper.<LootPool[]>getAsObject(jsonObject5, "pools", new LootPool[0], jsonDeserializationContext, LootPool[].class);
            LootContextParamSet ctv7 = null;
            if (jsonObject5.has("type")) {
                final String string8 = GsonHelper.getAsString(jsonObject5, "type");
                ctv7 = LootContextParamSets.get(new ResourceLocation(string8));
            }
            final LootItemFunction[] arr7 = GsonHelper.<LootItemFunction[]>getAsObject(jsonObject5, "functions", new LootItemFunction[0], jsonDeserializationContext, LootItemFunction[].class);
            return new LootTable((ctv7 != null) ? ctv7 : LootContextParamSets.ALL_PARAMS, arr6, arr7, null);
        }
        
        @Override
        public JsonElement serialize(final LootTable crx, final Type type, final JsonSerializationContext jsonSerializationContext) {
            final JsonObject jsonObject5 = new JsonObject();
            if (crx.paramSet != LootTable.DEFAULT_PARAM_SET) {
                final ResourceLocation sm6 = LootContextParamSets.getKey(crx.paramSet);
                if (sm6 != null) {
                    jsonObject5.addProperty("type", sm6.toString());
                }
                else {
                    LootTable.LOGGER.warn("Failed to find id for param set " + crx.paramSet);
                }
            }
            if (crx.pools.length > 0) {
                jsonObject5.add("pools", jsonSerializationContext.serialize(crx.pools));
            }
            if (!ArrayUtils.isEmpty((Object[])crx.functions)) {
                jsonObject5.add("functions", jsonSerializationContext.serialize(crx.functions));
            }
            return jsonObject5;
        }
    }
}
